﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data.SqlClient;
using System.Linq;
using System.Text;

namespace TestAdminSQLTable
{
    [Serializable]
    public class SQLTable
    {
        #region Constructors
        public SQLTable()
        {
        }
        #endregion
        #region Private Fields
        private int _ID;
        private string _TableName;
        private string _Name;
        private string _SelectCommand;
        private string _UpdateCommand;
        private string _DeleteCommand;
        private string _InsertCommand;
        private string _KeyFields;
        #endregion
        #region Public Properties
        public int ID
        {
            get { return _ID; }
            set { _ID = value; }
        }
        public string TableName
        {
            get { return _TableName; }
            set { _TableName = value; }
        }
        public string Name
        {
            get { return _Name; }
            set { _Name = value; }
        }
        public string SelectCommand
        {
            get { return _SelectCommand; }
            set { _SelectCommand = value; }
        }
        public string UpdateCommand
        {
            get { return _UpdateCommand; }
            set { _UpdateCommand = value; }
        }
        public string DeleteCommand
        {
            get { return _DeleteCommand; }
            set { _DeleteCommand = value; }
        }
        public string InsertCommand
        {
            get { return _InsertCommand; }
            set { _InsertCommand = value; }
        }
        public string KeyFields
        {
            get { return _KeyFields; }
            set { _KeyFields = value; }
        }
        #endregion

        public static List<SQLTable> GetAdministrableTables()
        {
            List<SQLTable> result = new List<SQLTable>();
            using (SqlConnection cn = new SqlConnection(ConfigurationManager.ConnectionStrings["MyConnectionString"].ConnectionString))
            {
                using (SqlCommand cm = new SqlCommand("SELECT * FROM DataType ORDER BY Name", cn))
                {
                    cn.Open();
                    using (SqlDataReader dr = cm.ExecuteReader())
                    {
                        while (dr.Read())
                        {
                            SQLTable t = new SQLTable();
                            t.ID = (int)dr["ID"];
                            t.TableName = dr["TableName"].ToString();
                            t.Name = dr["Name"].ToString();
                            t.SelectCommand = dr["SelectCommand"] != DBNull.Value ? dr["SelectCommand"].ToString().Trim() : null;
                            t.DeleteCommand = dr["DeleteCommand"] != DBNull.Value ? dr["DeleteCommand"].ToString().Trim() : null;
                            t.InsertCommand = dr["InsertCommand"] != DBNull.Value ? dr["InsertCommand"].ToString().Trim() : null;
                            t.UpdateCommand = dr["UpdateCommand"] != DBNull.Value ? dr["UpdateCommand"].ToString().Trim() : null;
                            t.KeyFields = dr["KeyFields"] != DBNull.Value ? dr["KeyFields"].ToString() : null;
                            result.Add(t);
                        }
                    }
                }
            }
            return result;
        }
        public List<SQLColumn> GetColumns()
        {
            List<SQLColumn> result = new List<SQLColumn>();
            using (SqlConnection cn = new SqlConnection(ConfigurationManager.ConnectionStrings["MyConnectionString"].ConnectionString))
            {
                using (SqlCommand cm = new SqlCommand("SELECT * FROM [TableSchema] WHERE DataTypeID='" + this.ID + "' ORDER BY OrdinalPosition", cn))
                {
                    cn.Open();
                    using (SqlDataReader dr = cm.ExecuteReader())
                    {
                        while (dr.Read())
                        {
                            SQLColumn t = new SQLColumn();
                            t.ColumnName = dr["ColumnName"].ToString();
                            t.DisplayColumnName = dr["DisplayColumnName"].ToString();
                            if (dr["ForeignTable"] != DBNull.Value)
                                t.ForeignTable = dr["ForeignTable"].ToString();
                            if (dr["ForeignTextColumn"] != DBNull.Value)
                                t.ForeignTextColumn = dr["ForeignTextColumn"].ToString();
                            if (dr["ForeignValueColumn"] != DBNull.Value)
                                t.ForeignValueColumn = dr["ForeignValueColumn"].ToString();
                            if (dr["FieldType"] != DBNull.Value)
                                t.FieldType = dr["FieldType"].ToString();

                            t.InsertParam = dr["InsertParam"] != DBNull.Value ? (bool)dr["InsertParam"] : false;
                            t.UpdateParam = dr["UpdateParam"] != DBNull.Value ? (bool)dr["UpdateParam"] : false;
                            t.DeleteParam = dr["DeleteParam"] != DBNull.Value ? (bool)dr["DeleteParam"] : false;
                            t.Visible = dr["Visible"] != DBNull.Value ? (bool)dr["Visible"] : false;
                            t.ReadOnly = dr["ReadOnly"] != DBNull.Value ? (bool)dr["ReadOnly"] : false;
                            result.Add(t);
                        }
                    }
                }
            }
            return result;
        }
    }


    public class SQLColumn
    {
        #region Constructors
        public SQLColumn()
        {
        }
        #endregion
        #region Private Fields
        private string _ColumnName;
        private string _DisplayColumnName;
        private int _OrdinalPosition;
        private bool _Visible;
        private bool _ReadOnly;
        private string _ForeignTable;
        private string _ForeignTextColumn;
        private string _ForeignValueColumn;
        private bool _InsertParam;
        private bool _UpdateParam;
        private bool _DeleteParam;
        #endregion
        #region Public Properties
        public string ColumnName
        {
            get { return _ColumnName; }
            set { _ColumnName = value; }
        }
        public string DisplayColumnName
        {
            get { return string.IsNullOrEmpty(_DisplayColumnName) ? _ColumnName : _DisplayColumnName; }
            set { _DisplayColumnName = value; }
        }
        public string ForeignTable
        {
            get { return _ForeignTable; }
            set { _ForeignTable = value; }
        }
        public string ForeignTextColumn
        {
            get { return _ForeignTextColumn; }
            set { _ForeignTextColumn = value; }
        }
        public string ForeignValueColumn
        {
            get { return _ForeignValueColumn; }
            set { _ForeignValueColumn = value; }
        }
        public int OrdinalPosition
        {
            get { return _OrdinalPosition; }
            set { _OrdinalPosition = value; }
        }
        public bool Visible
        {
            get { return _Visible; }
            set { _Visible = value; }
        }
        public bool ReadOnly
        {
            get { return _ReadOnly; }
            set { _ReadOnly = value; }
        }
        public bool InsertParam
        {
            get { return _InsertParam; }
            set { _InsertParam = value; }
        }
        public bool UpdateParam
        {
            get { return _UpdateParam; }
            set { _UpdateParam = value; }
        }
        public bool DeleteParam
        {
            get { return _DeleteParam; }
            set { _DeleteParam = value; }
        }
        public string FieldType { get; set; }
        #endregion
    }
}
